@@ -2,6 +2,7 @@ from __future__ import annotations
 
 import os
 import sys
+import sysconfig
 from contextlib import contextmanager, nullcontext, suppress
 from functools import cached_property
 from os.path import isabs
@@ -15,6 +16,8 @@ from hatch.utils.shells import ShellManager
 from hatch.utils.structures import EnvVars
 from hatch.venv.core import UVVirtualEnv, VirtualEnv
 
+FREETHREADED_BUILD = bool(sysconfig.get_config_var("Py_GIL_DISABLED"))
+
 if TYPE_CHECKING:
     from collections.abc import Callable, Iterable
 
@@ -118,7 +121,7 @@ class VirtualEnvironment(EnvironmentInterface):
             or is_default_environment(env_name, self.app.project.config.internal_envs[env_name])
         ):
             uv_env = self.app.project.get_environment(env_name)
-            self.app.project.prepare_environment(uv_env)
+            self.app.project.prepare_environment(uv_env, keep_env=bool(os.environ.get(AppEnvVars.KEEP_ENV)))
             with uv_env:
                 return self.platform.modules.shutil.which("uv")
 
@@ -198,37 +201,42 @@ class VirtualEnvironment(EnvironmentInterface):
 
     def sync_dependencies(self):
         with self.safe_activation():
-            # Install workspace members first as editable (already exists)
-            workspace_deps = [str(dep.path) for dep in self.local_dependencies_complex if dep.path]
-            if workspace_deps:
-                editable_args = []
-                for dep_path in workspace_deps:
-                    editable_args.extend(["--editable", dep_path])
-                self.platform.check_command(self.construct_pip_install_command(editable_args))
+            # If we do not have missing dependencies we should not sync
+            if not self.missing_dependencies:
+                return
 
-            # Separate remaining dependencies by type and filter conflicts
-            standard_dependencies: list[str] = []
-            editable_dependencies: list[str] = []
+            all_install_args = []
+
+            workspace_deps = [dep for dep in self.local_dependencies_complex if dep.path]
+            workspace_names = {dep.name.lower() for dep in self.local_dependencies_complex if dep.path}
+
+            for dep in workspace_deps:
+                if dep.editable:
+                    all_install_args.extend(["--editable", dep.path])
+                else:
+                    all_install_args.append(dep.path)
+
+            standard_dependencies = []
+
+            user_editable_dependencies = []
 
             for dependency in self.missing_dependencies:
-                # Skip if workspace member (already installed above)
-                if dependency.name.lower() in workspace_deps:
+                # Skip if already handled as workspace dependency
+                if dependency.name.lower() in workspace_names:
                     continue
 
-                if not dependency.editable or dependency.path is None:
-                    standard_dependencies.append(str(dependency))
+                if dependency.editable and dependency.path is not None:
+                    user_editable_dependencies.append(str(dependency.path))
                 else:
-                    editable_dependencies.append(str(dependency.path))
+                    standard_dependencies.append(str(dependency))
 
-            # Install other dependencies
-            if standard_dependencies:
-                self.platform.check_command(self.construct_pip_install_command(standard_dependencies))
+            all_install_args.extend(standard_dependencies)
 
-            if editable_dependencies:
-                editable_args = []
-                for dependency in editable_dependencies:
-                    editable_args.extend(["--editable", dependency])
-                self.platform.check_command(self.construct_pip_install_command(editable_args))
+            for dep_path in user_editable_dependencies:
+                all_install_args.extend(["--editable", dep_path])
+
+            if all_install_args:
+                self.platform.check_command(self.construct_pip_install_command(all_install_args))
 
     @contextmanager
     def command_context(self):
@@ -277,7 +285,12 @@ class VirtualEnvironment(EnvironmentInterface):
 
     @cached_property
     def _preferred_python_version(self):
-        return f"{sys.version_info.major}.{sys.version_info.minor}"
+        version = f"{sys.version_info.major}.{sys.version_info.minor}"
+
+        if FREETHREADED_BUILD:
+            version += "t"
+
+        return version
 
     @cached_property
     def parent_python(self):
